@@ -48,6 +48,35 @@ cmd_show_neighbors(struct lldpctl_conn_t *conn, struct writer *w,
 	return 1;
 }
 
+/**
+ * Show interfaces.
+ *
+ * The environment will contain the following keys:
+ *  - C{ports} list of ports we want to restrict showing.
+ *  - C{hidden} if we should show hidden ports.
+ *  - C{summary} if we want to show only a summary
+ *  - C{detailed} for a detailed overview
+ */
+static int
+cmd_show_interfaces(struct lldpctl_conn_t *conn, struct writer *w,
+    struct cmd_env *env, void *arg)
+{
+	log_debug("lldpctl", "show interfaces data (%s) %s hidden interfaces",
+	    cmdenv_get(env, "summary")?"summary":
+	    cmdenv_get(env, "detailed")?"detailed":
+	    "normal", cmdenv_get(env, "hidden")?"with":"without");
+	if (cmdenv_get(env, "ports"))
+		log_debug("lldpctl", "restrict to the following ports: %s",
+		    cmdenv_get(env, "ports"));
+
+	display_local_interfaces(conn, w, env, !!cmdenv_get(env, "hidden"),
+	    cmdenv_get(env, "summary")?DISPLAY_BRIEF:
+	    cmdenv_get(env, "detailed")?DISPLAY_DETAILS:
+	    DISPLAY_NORMAL);
+
+	return 1;
+}
+
 /**
  * Show chassis.
  *
@@ -286,6 +315,12 @@ register_commands_show(struct cmd_node *root)
 		"Show neighbors data",
 		NULL, NULL, NULL);
 
+	struct cmd_node *interfaces = commands_new(
+		show,
+		"interfaces",
+		"Show interfaces data",
+		NULL, NULL, NULL);
+
 	struct cmd_node *chassis = commands_new(
 		show,
 		"chassis",
@@ -306,6 +341,15 @@ register_commands_show(struct cmd_node *root)
 
 	register_common_commands(neighbors, 1);
 
+	/* Interfaces data */
+	commands_new(interfaces,
+	    NEWLINE,
+	    "Show interfaces data",
+	    NULL, cmd_show_interfaces, NULL);
+
+	cmd_restrict_ports(interfaces);
+	register_common_commands(interfaces, 0);
+
 	/* Chassis data */
 	commands_new(chassis,
 	    NEWLINE,
