@@ -19,6 +19,7 @@
 
 #include <stdio.h>
 #include <unistd.h>
+#include <inttypes.h>
 #include <errno.h>
 #include <sys/ioctl.h>
 #if defined(__clang__)
@@ -277,12 +278,64 @@ iflinux_is_bond(struct lldpd *cfg,
 	return 0;
 }
 
-static void
-iflinux_get_permanent_mac(struct lldpd *cfg,
+/**
+ * Get permanent MAC from ethtool.
+ *
+ * Return 0 on success, -1 on error.
+ */
+static int
+iflinux_get_permanent_mac_ethtool(struct lldpd *cfg,
     struct interfaces_device_list *interfaces,
     struct interfaces_device *iface)
 {
-	struct interfaces_device *master;
+	struct ifreq ifr = {};
+	union {
+		struct ethtool_perm_addr addr;
+		/* cppcheck-suppress unusedStructMember */
+		char u8[sizeof(struct ethtool_perm_addr) + ETHER_ADDR_LEN];
+	} epaddr;
+
+	strlcpy(ifr.ifr_name, iface->name, sizeof(ifr.ifr_name));
+	epaddr.addr.cmd = ETHTOOL_GPERMADDR;
+	epaddr.addr.size = ETHER_ADDR_LEN;
+	ifr.ifr_data = (caddr_t)&epaddr.addr;
+	if (ioctl(cfg->g_sock, SIOCETHTOOL, &ifr) == -1) {
+		static int once = 0;
+		if (errno == EPERM && !once) {
+			log_warn("interfaces",
+			    "no permission to get permanent MAC address for %s (requires 2.6.19+)",
+			    iface->name);
+			once = 1;
+			return -1;
+		}
+		if (errno != EPERM)
+			log_warnx("interfaces", "cannot get permanent MAC address for %s",
+			    iface->name);
+		return -1;
+	}
+	if (epaddr.addr.data[0] != 0 ||
+	    epaddr.addr.data[1] != 0 ||
+	    epaddr.addr.data[2] != 0 ||
+	    epaddr.addr.data[3] != 0 ||
+	    epaddr.addr.data[4] != 0 ||
+	    epaddr.addr.data[5] != 0 ||
+	    epaddr.addr.data[6] != 0) {
+		memcpy(iface->address, epaddr.addr.data, ETHER_ADDR_LEN);
+		return 0;
+	}
+	log_debug("interfaces", "cannot get permanent MAC for %s", iface->name);
+	return -1;
+}
+
+/**
+ * Get permanent MAC address for a bond device.
+ */
+static void
+iflinux_get_permanent_mac_bond(struct lldpd *cfg,
+    struct interfaces_device_list *interfaces,
+    struct interfaces_device *iface)
+{
+	struct interfaces_device *master = iface->upper;
 	int f, state = 0;
 	FILE *netbond;
 	const char *slaveif = "Slave Interface: ";
@@ -291,9 +344,6 @@ iflinux_get_permanent_mac(struct lldpd *cfg,
 	char path[SYSFS_PATH_MAX];
 	char line[100];
 
-	if ((master = iface->upper) == NULL || master->type != IFACE_BOND_T)
-		return;
-
 	/* We have a bond, we need to query it to get real MAC addresses */
 	if (snprintf(path, SYSFS_PATH_MAX, "/proc/net/bonding/%s",
 		master->name) >= SYSFS_PATH_MAX) {
@@ -311,7 +361,7 @@ iflinux_get_permanent_mac(struct lldpd *cfg,
 	if (f < 0) {
 		log_warnx("interfaces",
 		    "unable to get permanent MAC address for %s",
-		    master->name);
+		    iface->name);
 		return;
 	}
 	if ((netbond = fdopen(f, "r")) == NULL) {
@@ -359,11 +409,43 @@ iflinux_get_permanent_mac(struct lldpd *cfg,
 			break;
 		}
 	}
-	log_warnx("interfaces", "unable to find real mac address for %s",
+	log_warnx("interfaces", "unable to find real MAC address for enslaved %s",
 	    iface->name);
 	fclose(netbond);
 }
 
+/**
+ * Get permanent MAC.
+ */
+static void
+iflinux_get_permanent_mac(struct lldpd *cfg,
+    struct interfaces_device_list *interfaces,
+    struct interfaces_device *iface)
+{
+	struct interfaces_device *master = iface->upper;
+
+	if (master == NULL || master->type != IFACE_BOND_T)
+		return;
+	if (iflinux_get_permanent_mac_ethtool(cfg, interfaces, iface) == -1 &&
+	    (master->driver == NULL || !strcmp(master->driver, "bonding")))
+		/* Fallback to old method for a bond */
+		iflinux_get_permanent_mac_bond(cfg, interfaces, iface);
+}
+
+#ifdef ENABLE_DOT3
+#define ETHTOOL_LINK_MODE_MASK_MAX_KERNEL_NU32 (SCHAR_MAX)
+#define ETHTOOL_DECLARE_LINK_MODE_MASK(name)			\
+	uint32_t name[ETHTOOL_LINK_MODE_MASK_MAX_KERNEL_NU32]
+
+struct ethtool_link_usettings {
+	struct ethtool_link_settings base;
+	struct {
+		ETHTOOL_DECLARE_LINK_MODE_MASK(supported);
+		ETHTOOL_DECLARE_LINK_MODE_MASK(advertising);
+		ETHTOOL_DECLARE_LINK_MODE_MASK(lp_advertising);
+	} link_modes;
+};
+
 static inline int
 iflinux_ethtool_link_mode_test_bit(unsigned int nr, const uint32_t *mask)
 {
@@ -391,12 +473,102 @@ iflinux_ethtool_link_mode_is_empty(const uint32_t *mask)
 	return 1;
 }
 
+static int
+iflinux_ethtool_glink(struct lldpd *cfg, const char *ifname, struct ethtool_link_usettings *uset) {
+	int rc;
+
+	/* Try with ETHTOOL_GLINKSETTINGS first */
+	struct {
+		struct ethtool_link_settings req;
+		uint32_t link_mode_data[3 * ETHTOOL_LINK_MODE_MASK_MAX_KERNEL_NU32];
+	} ecmd;
+	static int8_t nwords = 0;
+	struct ifreq ifr = {};
+	strlcpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
+
+	if (nwords == 0) {
+		/* Do a handshake first. We assume that this is device-independant. */
+		memset(&ecmd, 0, sizeof(ecmd));
+		ecmd.req.cmd = ETHTOOL_GLINKSETTINGS;
+		ifr.ifr_data = (caddr_t)&ecmd;
+		rc = ioctl(cfg->g_sock, SIOCETHTOOL, &ifr);
+		if (rc == 0) {
+			nwords = -ecmd.req.link_mode_masks_nwords;
+			log_debug("interfaces", "glinksettings nwords is %" PRId8, nwords);
+		} else {
+			static int once = 0;
+			if (errno == EPERM && !once) {
+				log_warn("interfaces",
+				    "cannot get ethtool link information "
+				    "with GLINKSETTINGS (requires 4.9+)");
+				once = 1;
+			}
+			nwords = -1;
+		}
+	}
+	if (nwords > 0) {
+		memset(&ecmd, 0, sizeof(ecmd));
+		ecmd.req.cmd = ETHTOOL_GLINKSETTINGS;
+		ecmd.req.link_mode_masks_nwords = nwords;
+		ifr.ifr_data = (caddr_t)&ecmd;
+		rc = ioctl(cfg->g_sock, SIOCETHTOOL, &ifr);
+		if (rc == 0) {
+			log_debug("interfaces", "got ethtool results for %s with GLINKSETTINGS",
+			    ifname);
+			memcpy(&uset->base, &ecmd.req, sizeof(uset->base));
+			unsigned int u32_offs = 0;
+			memcpy(uset->link_modes.supported,
+			    &ecmd.link_mode_data[u32_offs],
+			    4 * ecmd.req.link_mode_masks_nwords);
+			u32_offs += ecmd.req.link_mode_masks_nwords;
+			memcpy(uset->link_modes.advertising,
+			    &ecmd.link_mode_data[u32_offs],
+			    4 * ecmd.req.link_mode_masks_nwords);
+			u32_offs += ecmd.req.link_mode_masks_nwords;
+			memcpy(uset->link_modes.lp_advertising,
+			    &ecmd.link_mode_data[u32_offs],
+			    4 * ecmd.req.link_mode_masks_nwords);
+			goto end;
+		}
+	}
+
+	/* Try with ETHTOOL_GSET */
+	struct ethtool_cmd ethc;
+	memset(&ethc, 0, sizeof(ethc));
+	ethc.cmd = ETHTOOL_GSET;
+	ifr.ifr_data = (caddr_t)&ethc;
+	rc = ioctl(cfg->g_sock, SIOCETHTOOL, &ifr);
+	if (rc == 0) {
+		/* Do a partial copy (only what we need) */
+		log_debug("interfaces", "got ethtool results for %s with GSET",
+		    ifname);
+		memset(uset, 0, sizeof(*uset));
+		uset->base.cmd = ETHTOOL_GSET;
+		uset->base.link_mode_masks_nwords = 1;
+		uset->link_modes.supported[0] = ethc.supported;
+		uset->link_modes.advertising[0] = ethc.advertising;
+		uset->link_modes.lp_advertising[0] = ethc.lp_advertising;
+		uset->base.speed = (ethc.speed_hi << 16) | ethc.speed;
+		uset->base.duplex = ethc.duplex;
+		uset->base.port = ethc.port;
+		uset->base.autoneg = ethc.autoneg;
+	} else {
+		static int once = 0;
+		if (errno == EPERM && !once) {
+			log_warn("interfaces",
+			    "cannot get ethtool link information "
+			    "with GSET (requires 2.6.19+)");
+			once = 1;
+		}
+	}
+end:
+	return rc;
+}
 
 /* Fill up MAC/PHY for a given hardware port */
 static void
-iflinux_macphy(struct lldpd_hardware *hardware)
+iflinux_macphy(struct lldpd *cfg, struct lldpd_hardware *hardware)
 {
-#ifdef ENABLE_DOT3
 	struct ethtool_link_usettings uset;
 	struct lldpd_port *port = &hardware->h_lport;
 	int j;
@@ -414,7 +586,7 @@ iflinux_macphy(struct lldpd_hardware *hardware)
 
 	log_debug("interfaces", "ask ethtool for the appropriate MAC/PHY for %s",
 	    hardware->h_ifname);
-	if (priv_ethtool(hardware->h_ifname, &uset) == 0) {
+	if (iflinux_ethtool_glink(cfg, hardware->h_ifname, &uset) == 0) {
 		port->p_macphy.autoneg_support = iflinux_ethtool_link_mode_test_bit(
 			ETHTOOL_LINK_MODE_Autoneg_BIT, uset.link_modes.supported);
 		port->p_macphy.autoneg_enabled = (uset.base.autoneg == AUTONEG_DISABLE) ? 0 : 1;
@@ -479,15 +651,23 @@ iflinux_macphy(struct lldpd_hardware *hardware)
 			// Same kind of approximation.
 			port->p_macphy.mau_type = (uset.base.port == PORT_FIBRE) ? \
 			    LLDP_DOT3_MAU_40GBASELR4 : LLDP_DOT3_MAU_40GBASECR4;
+			break;
 		case SPEED_100000:
 			// Ditto
 			port->p_macphy.mau_type = (uset.base.port == PORT_FIBRE) ? \
 			    LLDP_DOT3_MAU_100GBASELR4 : LLDP_DOT3_MAU_100GBASECR10;
+			break;
 		}
 		if (uset.base.port == PORT_AUI) port->p_macphy.mau_type = LLDP_DOT3_MAU_AUI;
 	}
-#endif
 }
+#else /* ENABLE_DOT3 */
+static void
+iflinux_macphy(struct lldpd *cfg, struct lldpd_hardware *hardware)
+{
+}
+#endif /* ENABLE_DOT3 */
+
 
 struct bond_master {
 	char name[IFNAMSIZ];
@@ -863,7 +1043,7 @@ interfaces_update(struct lldpd *cfg)
 	/* Mac/PHY */
 	TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries) {
 		if (!hardware->h_flags) continue;
-		iflinux_macphy(hardware);
+		iflinux_macphy(cfg, hardware);
 		interfaces_helper_promisc(cfg, hardware);
 	}
 }
