@@ -47,7 +47,7 @@
 #define LLDP_PORTID_SUBTYPE_LOCAL	7
 #define LLDP_PORTID_SUBTYPE_MAX		LLDP_PORTID_SUBTYPE_LOCAL
 
-/* Operational MAU Type field, from RFC 3636 (see IANAifMauTypeListBits) */
+/* Operational MAU Type field. See: https://www.iana.org/assignments/ianamau-mib/ianamau-mib */
 #define LLDP_DOT3_MAU_AUI		1
 #define LLDP_DOT3_MAU_10BASE5		2
 #define LLDP_DOT3_MAU_FOIRL		3
@@ -101,6 +101,31 @@
 #define LLDP_DOT3_MAU_1000BASEPX10U	51
 #define LLDP_DOT3_MAU_1000BASEPX20D	52
 #define LLDP_DOT3_MAU_1000BASEPX20U	53
+#define LLDP_DOT3_MAU_10GBASET		54
+#define LLDP_DOT3_MAU_10GBASELRM	55
+#define LLDP_DOT3_MAU_1000BASEKX	56
+#define LLDP_DOT3_MAU_10GBASEKX4	57
+#define LLDP_DOT3_MAU_10GBASEKR		58
+#define LLDP_DOT3_MAU_10G1GBASEPRXD1	59
+#define LLDP_DOT3_MAU_10G1GBASEPRXD2	60
+#define LLDP_DOT3_MAU_10G1GBASEPRXD3	61
+#define LLDP_DOT3_MAU_10G1GBASEPRXU1	62
+#define LLDP_DOT3_MAU_10G1GBASEPRXU2	63
+#define LLDP_DOT3_MAU_10G1GBASEPRXU3	64
+#define LLDP_DOT3_MAU_10GBASEPRD1	65
+#define LLDP_DOT3_MAU_10GBASEPRD2	66
+#define LLDP_DOT3_MAU_10GBASEPRD3	67
+#define LLDP_DOT3_MAU_10GBASEPRU1	68
+#define LLDP_DOT3_MAU_10GBASEPRU3	69
+#define LLDP_DOT3_MAU_40GBASEKR4	70
+#define LLDP_DOT3_MAU_40GBASECR4	71
+#define LLDP_DOT3_MAU_40GBASESR4	72
+#define LLDP_DOT3_MAU_40GBASEFR		73
+#define LLDP_DOT3_MAU_40GBASELR4	74
+#define LLDP_DOT3_MAU_100GBASECR10	75
+#define LLDP_DOT3_MAU_100GBASESR10	76
+#define LLDP_DOT3_MAU_100GBASELR4	77
+#define LLDP_DOT3_MAU_100GBASEER4	78
 
 /* Dot3 Power Devicetype */
 #define LLDP_DOT3_POWER_PSE	1
@@ -129,7 +154,9 @@
 #define LLDP_DOT3_POWER_PRIO_HIGH	2
 #define LLDP_DOT3_POWER_PRIO_LOW	3
 
-/* PMD Auto-Negotiation Advertised Capability field, from RFC 3636 (see IANAifMauAutoNegCapBits) */
+/* PMD Auto-Negotiation Advertised Capability field, from RFC 3636 (see
+ * IANAifMauAutoNegCapBits). Unfortunately, we are limited to two bytes, so
+ * higher speed capabilities will map to "other". */
 #define LLDP_DOT3_LINK_AUTONEG_OTHER		0x8000
 #define LLDP_DOT3_LINK_AUTONEG_10BASE_T		0x4000
 #define LLDP_DOT3_LINK_AUTONEG_10BASET_FD	0x2000
